#include "libgraph.h"
#include "image.h"
#include <alloc.h>


value gr_image_size(im)	/* ML */
  value im;
{
  value res;
  res = alloc_tuple(2);
#ifdef DEBUG_IMG
  fprintf(stderr,"Caml: image_size: %d %d.\n",Width_im(im),Height_im(im));
#endif

  Field(res, 0) = Val_int( Width_im(im) );
  Field(res, 1) = Val_int( Height_im(im) );

  return res;
}




static void gr_free_image(im)
     value im;
{
  XFreePixmap(grdisplay, Data_im(im));
  if (Mask_im(im) != None) XFreePixmap(grdisplay, Mask_im(im));
}

value gr_new_image(w, h)
     int w, h;
{
  value res = alloc_shr(Grimage_wosize, Final_tag);

#ifdef DEBUG_IMG
  fprintf(stderr,"Caml: gr_new_image: %d, %d.\n",w,h);
#endif

  Final_fun(res) = gr_free_image;
  Width_im(res) = w;
  Height_im(res) = h;
  Data_im(res) = XCreatePixmap(grdisplay, grwindows[0].win, w+1, h, 
                               XDefaultDepth(grdisplay, grscreen));
  Mask_im(res) = None;
/*  XFlush(grdisplay); */
  return res;
}

value gr_create_image(vw, vh)
     value vw, vh;
{
  gr_check_open();
  return gr_new_image(Int_val(vw), Int_val(vh));
}

value gr_blit_image(im, vx, vy)
     value im, vx, vy;
{
  int x = Int_val(vx);
  int y = Int_val(vy);
#ifdef DEBUG_IMG
  fprintf(stderr,"Caml: blit_image: %d, %d.\n",x,y);
#endif
  if(ValidWindow==FALSE) return Val_unit;
  gr_check_open();
  XCopyArea(grdisplay, grwindows[current_window].win, Data_im(im), 
            grwindows[current_window].gc,
            x, Wcvt(y) + 1 - Height_im(im),
            Width_im(im), Height_im(im),
            0, 0);
  return Val_unit;
}

value gr_draw_image(im, vx, vy)
     value im, vx, vy;
{
  int x = Int_val(vx);
  int y = Int_val(vy);
  int wy = Wcvt(y) + 1 - Height_im(im);
#ifdef DEBUG_IMG
  fprintf(stderr,"draw_image: %d, %d.\n",x,y);
#endif

  if(ValidWindow==FALSE) return Val_unit;
  gr_check_open();
  if (Mask_im(im) != None) 
  {
    XSetClipOrigin(grdisplay, grwindows[current_window].gc, x, wy);
    XSetClipMask(grdisplay, grwindows[current_window].gc, Mask_im(im));
  }
  XCopyArea(grdisplay, Data_im(im), grwindows[current_window].win, 
            grwindows[current_window].gc,
            0, 0,
            Width_im(im), Height_im(im),
            x, wy);
  if (Mask_im(im) != None) 
    XSetClipMask(grdisplay, grwindows[current_window].gc, None);

/*  XFlush(grdisplay); */
  return Val_unit;
}


value gr_plot_image(im, vx, vy, col)
     value im, vx, vy, col;
{
  int x = Int_val(vx);
  int y = Int_val(vy);
  int color = Int_val(col);
  int i, j;
  int width, height;
  GC gc_mask,gc_data;

#ifdef DEBUG_IMG
  fprintf(stderr,"Caml: plot_image at %d, %d.\n",x,y);
#endif

  width = Width_im(im);
  height = Height_im(im);

  gr_check_open();

  gc_data = XCreateGC(grdisplay, Data_im(im), 0, NULL);

  if(Mask_im(im)!=None) 
  { 
    gc_mask = XCreateGC(grdisplay, Mask_im(im), 0, NULL);

    if (color == Transparent)
    {
      XSetForeground(grdisplay, gc_mask, 0);
      XDrawPoint(grdisplay, Mask_im(im), gc_mask, x, y);
      color = 0;
    }
    else  
    {
      XSetForeground(grdisplay, gc_mask, 1);
      XDrawPoint(grdisplay, Mask_im(im), gc_mask, x, y);
     }
  }
  else if (color == Transparent)
       {
         Mask_im(im) = XCreatePixmap(grdisplay, grwindows[0].win,
                                     width, height, 1);
         gc_mask = XCreateGC(grdisplay, Mask_im(im), 0, NULL);
         XSetForeground(grdisplay, gc_mask, 1);

         XFillRectangle(grdisplay, Mask_im(im), gc_mask,
                   0, 0, width, height);

         XSetForeground(grdisplay, gc_mask, 1);
         XDrawPoint(grdisplay, Mask_im(im), gc_mask, x, y);
         color = 0;
       }
  color = gr_pixel_rgb(color);
  XSetForeground(grdisplay, gc_data, color);
  XDrawPoint(grdisplay, Data_im(im), gc_data, x, y);
  XFreeGC(grdisplay, gc_data);

  if (Mask_im(im) != None)  XFreeGC(grdisplay, gc_mask);

  return Val_unit;
}


int get_pixel_color(im , x, y)
     Pixmap im;
     int x, y;
{
  XImage *xim;
  int val;

  if(ValidWindow==FALSE) return Val_int(0);
  xim = XGetImage(grdisplay, im, x, y, 1, 1, (-1), ZPixmap);
  val = XGetPixel(xim, 0, 0);
  XDestroyImage(xim);
  return val;
}



value gr_pixel_image(im, vx, vy)
     value im, vx, vy;
{
  int x = Int_val(vx);
  int y = Int_val(vy);
  int col,mask_col;


#ifdef DEBUG_IMG
  fprintf(stderr,"Caml: pixel_image at %d, %d.\n",x,y);
#endif

  gr_check_open();


  col = get_pixel_color(Data_im(im), x, y);
  col = gr_rgb_pixel(col);
  if(Mask_im(im)!=None) 
  { 
    mask_col=get_pixel_color(Mask_im(im), x, y);
    if(mask_col==0) col=Transparent;
  }

  return Val_int(col);
}


            
